/**
 * Define api of apeman-api-path
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aslogger = require('aslogger')
const path = require('path')
const spec = require('./spec')

let cwd = () => process.cwd()

/** @lends define */
function define (options = {}) {
  let {
    logging = false,
    root = cwd()
  } = options
  let logger = aslogger({ disabled: !logging })
  return {

    /**
     * Test the reachability of the api.
     * @method ping
     * @param {string} [pong='pong'] - Pong message to return
     * @returns {string} - Pong message
     */
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    /**
     * Resolve directory name
     * @method dirname
     * @param {string} filename - Filename to resolve
     * @returns {Promise}
     */
    dirname (filename) {
      const s = this
      let args = [ ...arguments ]
      return co(function * () {
        let dirname = path.dirname(...args)
        return yield s.resolve(dirname)
      })
    },

    /**
     * Get base name
     * @method basename
     * @param {string} filename - Filename to resolve
     * @param {string} [extname] - Extension name to strip off.
     * @returns {Promise}
     */
    basename (filename, extname) {
      const s = this
      let args = [ ...arguments ]
      return co(function * () {
        return path.basename(...args)
      })
    },

    /**
     * Resolve relative path
     * @method relative
     * @param {string} from - Relative filename from
     * @param {string} to - Relative filename to
     * @returns {Promise}
     */
    relative (from, to) {
      const s = this
      return co(function * () {
        from = yield s.resolve(from)
        to = yield s.resolve(to)
        return path.relative(from, to)
      })
    },

    /**
     * Resolve a file path
     * @method resolve
     * @param {...string} filenames - Filenames to resolve
     * @returns {Promise}
     */
    resolve (...filenames) {
      const s = this
      return co(function * () {
        let resolved = path.resolve(root, ...filenames)
        let isTraversal = resolved.indexOf(root) !== 0
        if (isTraversal) {
          throw new Error(`[apeman-api-path] DIRECTORY_TRAVERSAL_DETECTED ( passed: "${resolved}", allowed: "${root}"`)
        }
        return path.relative(cwd(), resolved)
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: spec
  }
}

module.exports = define
